/*
   Source for x86 emulator IdaPro plugin
   File: hooklist.cpp
   Copyright (c) 2004, Chris Eagle
   
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option) 
   any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
   more details.
   
   You should have received a copy of the GNU General Public License along with 
   this program; if not, write to the Free Software Foundation, Inc., 59 Temple 
   Place, Suite 330, Boston, MA 02111-1307 USA
*/
#include <stdlib.h>
#include <string.h>
#include "hooklist.h"

#ifndef NULL
#define NULL 0
#endif

static HookNode *hookList = NULL;

HookNode::HookNode(char *fName, unsigned int addr, hookfunc func, HookNode *nxt) :
   funcAddr(addr), func(func), next(nxt) {
   funcName = strdup(fName);
}

HookNode::~HookNode() {
   if (next) delete next;
   free(funcName);
}

void addHook(char *fName, unsigned int funcAddr, hookfunc func) {
   hookList = new HookNode(fName, funcAddr, func, hookList);
}

void removeHook(unsigned int funcAddr) {
   HookNode *prev = NULL, *curr = hookList;
   while (curr) {
      if (curr->funcAddr == funcAddr) {
         if (prev) {
            prev->next = curr->next;
         }
         else {
            hookList = curr->next;
         }
         free(curr->funcName);
         delete curr;
         break;
      }
      prev = curr;
      curr = curr->next;
   }
}

hookfunc findHook(unsigned int funcAddr) {
   for (HookNode *n = hookList; n; n = n->next) {
      if (n->funcAddr == funcAddr) {
         return n->func;
      }
   }
   return NULL;
}

HookNode *find(unsigned int funcAddr) {
   for (HookNode *n = hookList; n; n = n->next) {
      if (n->funcAddr == funcAddr) {
         return n;
      }
   }
   return NULL;
}

HookNode *find(char *fName) {
   for (HookNode *n = hookList; n; n = n->next) {
      if (!strcmp(n->funcName, fName)) {
         return n;
      }
   }
   return NULL;
}

HookNode *getNext(HookNode *n) {
   return n ? n->next : hookList;
}

